@@ -37,30 +37,23 @@ class InstallWizard(QDialog):
         self.setMaximumSize(575, 400)
         self.setWindowTitle('Reddcoin Electrum')
         self.connect(self, QtCore.SIGNAL('accept'), self.accept)
-
         self.stack = QStackedLayout()
         self.setLayout(self.stack)
 
-
     def set_layout(self, layout):
         w = QWidget()
         w.setLayout(layout)
-        self.stack.setCurrentIndex(self.stack.addWidget(w))
-
+        self.stack.addWidget(w)
+        self.stack.setCurrentWidget(w)
 
     def restore_or_create(self):
-
         vbox = QVBoxLayout()
-
         main_label = QLabel(_("Reddcoin Electrum could not find an existing wallet."))
         vbox.addWidget(main_label)
-
         grid = QGridLayout()
         grid.setSpacing(5)
-
         gb1 = QGroupBox(_("What do you want to do?"))
         vbox.addWidget(gb1)
-
         b1 = QRadioButton(gb1)
         b1.setText(_("Create new wallet"))
         b1.setChecked(True)
@@ -104,7 +97,7 @@ class InstallWizard(QDialog):
 
         if not self.exec_():
             return None, None
-        
+
         action = 'create' if b1.isChecked() else 'restore'
         wallet_type = self.wallet_types[group2.checkedId()][0]
         return action, wallet_type
@@ -135,7 +128,7 @@ class InstallWizard(QDialog):
     def enter_seed_dialog(self, msg, sid, func=None):
         if func is None:
             func = self.is_any
-        vbox, seed_e = seed_dialog.enter_seed_box(msg, sid)
+        vbox, seed_e = seed_dialog.enter_seed_box(msg, self, sid)
         vbox.addStretch(1)
         hbox, button = ok_cancel_buttons2(self, _('Next'))
         vbox.addLayout(hbox)
@@ -191,7 +184,7 @@ class InstallWizard(QDialog):
 
         self.set_layout(vbox)
         if not self.exec_():
-            return 
+            return
         return map(lambda e: self.get_seed_text(e), entries)
 
 
@@ -218,7 +211,7 @@ class InstallWizard(QDialog):
         # skip this if config already exists
         if self.config.get('server') is not None:
             return
-        
+
         grid = QGridLayout()
         grid.setSpacing(5)
 
@@ -252,14 +245,14 @@ class InstallWizard(QDialog):
         self.set_layout(vbox)
         if not self.exec_():
             return
-        
+
         if b2.isChecked():
             return NetworkDialog(self.network, self.config, None).do_exec()
         else:
             self.config.set_key('auto_cycle', True, True)
             return
 
-        
+
     def show_message(self, msg, icon=None):
         vbox = QVBoxLayout()
         self.set_layout(vbox)
@@ -270,7 +263,7 @@ class InstallWizard(QDialog):
         vbox.addWidget(QLabel(msg))
         vbox.addStretch(1)
         vbox.addLayout(close_button(self, _('Next')))
-        if not self.exec_(): 
+        if not self.exec_():
             return None
 
 
@@ -309,7 +302,7 @@ class InstallWizard(QDialog):
         vbox.addWidget(label)
         vbox.addStretch(1)
         vbox.addLayout(ok_cancel_buttons(self, yes_label, no_label))
-        if not self.exec_(): 
+        if not self.exec_():
             return None
         return True
 
@@ -349,7 +342,7 @@ class InstallWizard(QDialog):
                 wallet_type = '2fa'
 
             if action == 'create':
-                self.storage.put('wallet_type', wallet_type)
+                self.storage.put('wallet_type', wallet_type, False)
 
         if action is None:
             return
@@ -407,7 +400,7 @@ class InstallWizard(QDialog):
 
             else:
                 f = run_hook('get_wizard_action', self, wallet, action)
-                if not f: 
+                if not f:
                     raise BaseException('unknown wizard action', action)
                 r = f(wallet, self)
                 if not r:
@@ -466,7 +459,7 @@ class InstallWizard(QDialog):
 
             elif t in ['2of2']:
                 r = self.multi_seed_dialog(1)
-                if not r: 
+                if not r:
                     return
                 text1, text2 = r
                 wallet = Wallet_2of2(self.storage)
@@ -496,7 +489,7 @@ class InstallWizard(QDialog):
 
             elif t in ['2of3']:
                 r = self.multi_seed_dialog(2)
-                if not r: 
+                if not r:
                     return
                 text1, text2, text3 = r
                 wallet = Wallet_2of3(self.storage)
@@ -541,5 +534,5 @@ class InstallWizard(QDialog):
 
             # create first keys offline
             self.waiting_dialog(wallet.synchronize)
-                
+
             return wallet
