@@ -57,7 +57,6 @@ paytomany_syntax = "paytomany <recipient> <amount> [<recipient> <amount> ...]\n<
 signmessage_syntax = 'signmessage <address> <message>\nIf you want to lead or end a message with spaces, or want double spaces inside the message make sure you quote the string. I.e. " Hello  This is a weird String "'
 verifymessage_syntax = 'verifymessage <address> <signature> <message>\nIf you want to lead or end a message with spaces, or want double spaces inside the message make sure you quote the string. I.e. " Hello  This is a weird String "'
 
-
 #                command
 #                                              requires_network
 #                                                     requires_wallet
@@ -93,10 +92,12 @@ register_command('payto',                5, 5, True,  True,  True,  'Create and
 register_command('paytomany',            4, 4, True,  True,  True,  'Create and broadcast a transaction.', paytomany_syntax, payto_options)
 register_command('password',             0, 0, False, True,  True,  'Change your password')
 register_command('restore',              0, 0, True,  True,  False, 'Restore a wallet', '', restore_options)
+register_command('searchcontacts',       1, 1, False, True,  False,  'Search through contacts, return matching entries', 'searchcontacts <query>')
 register_command('setconfig',            2, 2, False, False, False, 'Set a configuration variable', 'setconfig <name> <value>')
 register_command('setlabel',             2,-1, False, True,  False, 'Assign a label to an item', 'setlabel <tx_hash> <label>')
 register_command('sendrawtransaction',   1, 1, True,  False, False, 'Broadcasts a transaction to the network.', 'sendrawtransaction <tx in hexadecimal>')
-register_command('signrawtransaction',   1, 3, False, True,  True,  'Sign a serailized transaction','signrawtransaction <tx in hexadecimal>')
+register_command('signtxwithkey',        1, 3, False, False, False, 'Sign a serialized transaction with a key','signtxwithkey <tx> <key>')
+register_command('signtxwithwallet',     1, 3, False, True,  True,  'Sign a serialized transaction with a wallet','signtxwithwallet <tx>')
 register_command('signmessage',          2,-1, False, True,  True,  'Sign a message with a key', signmessage_syntax)
 register_command('unfreeze',             1, 1, False, True,  False, 'Unfreeze the funds at one of your wallet\'s address', 'unfreeze <address>')
 register_command('validateaddress',      1, 1, False, False, False, 'Check that the address is valid', 'validateaddress <address>')
@@ -163,9 +164,15 @@ class Commands:
         tx = Transaction(inputs, outputs)
         return tx
 
-    def signrawtransaction(self, raw_tx, private_keys):
+    def signtxwithkey(self, raw_tx, sec):
         tx = Transaction.deserialize(raw_tx)
-        self.wallet.signrawtransaction(tx, private_keys, self.password)
+        pubkey = bitcoin.public_key_from_private_key(sec)
+        tx.sign({ pubkey:sec })
+        return tx
+
+    def signtxwithwallet(self, raw_tx):
+        tx = Transaction.deserialize(raw_tx)
+        self.wallet.sign_transaction(tx, self.password)
         return tx
 
     def decoderawtransaction(self, raw):
@@ -343,6 +350,13 @@ class Commands:
             c[addr] = self.wallet.labels.get(addr)
         return c
 
+    def searchcontacts(self, query):
+        results = {}
+        for addr in self.wallet.addressbook:
+            if query.lower() in self.wallet.labels.get(addr).lower():
+                results[addr] = self.wallet.labels.get(addr)
+        return results
+
     def listaddresses(self, show_all=False, show_label=False):
         out = []
         for addr in self.wallet.addresses(True):
