@@ -67,18 +67,21 @@ def data_dir():
     else:
         return appdata_dir()
 
+def usr_share_dir():
+    return os.path.join(sys.prefix, "share")
 
 def appdata_dir():
     """Find the path to the application data directory; add an electrum folder and return path."""
     if platform.system() == "Windows":
         return os.path.join(os.environ["APPDATA"], "Reddcoin-Electrum")
     elif (platform.system() == "Linux" or
-          platform.system() == "Darwin" or
           platform.system() == "DragonFly" or
           platform.system() == "OpenBSD" or
           platform.system() == "FreeBSD" or
           platform.system() == "NetBSD"):
-        return os.path.join("/usr/share", "reddcoin-electrum")
+        return os.path.join(sys.prefix, "share", "electrum")
+    elif platform.system() == "Darwin":
+		return "/Library/Application Support/Electrum"
     else:
         raise Exception("Unknown Operating System")
 
@@ -186,7 +189,7 @@ def parse_URI(uri):
     valid_address = bitcoin.is_address(address)
 
     pq = urlparse.parse_qs(u.query)
-    
+
     for k, v in pq.items():
         if len(v)!=1:
             raise Exception('Duplicate Key', k)
@@ -206,7 +209,7 @@ def parse_URI(uri):
         label = pq['label'][0]
     if 'r' in pq:
         request_url = pq['r'][0]
-        
+
     if request_url != '':
         return address, amount, label, message, request_url
 
@@ -228,7 +231,7 @@ __builtin__.raw_input = raw_input
 
 def parse_json(message):
     n = message.find('\n')
-    if n==-1: 
+    if n==-1:
         return None, message
     try:
         j = json.loads(message[0:n])
